from itertools import islice
from typing import Any, Iterable


DictKeys = type({}.keys())
DictValues = type({}.values())
DictItems = type({}.items())


def repr_trunc(iterable: Iterable[Any]) -> str:
    """Returns a truncated `repr` for `iterable`, limited to the first 5 items."""
    itype = type(iterable)
    if itype is list:
        return f"[{_join_first_5_reprs(iterable)}]"
    elif itype is tuple:
        return f"({_join_first_5_reprs(iterable)})"
    elif itype is dict:
        joined = ", ".join(map(__join_dict_item, islice(iterable.items(), 5)))
        if len(iterable) > 5:
            joined += ", ..."
        return "{" + joined + "}"
    elif itype is DictKeys:
        return f"dict_keys([{_join_first_5_reprs(iterable)}])"
    elif itype is DictValues:
        return f"dict_values([{_join_first_5_reprs(iterable)}])"
    elif itype is DictItems:
        return f"dict_items([{_join_first_5_reprs(iterable)}])"
    elif itype is set:
        return "{" + _join_first_5_reprs(iterable) + "}"
    else:
        return repr(iterable)


def _join_first_5_reprs(iterable: Iterable[Any]) -> str:
    joined = ", ".join(map(repr, islice(iterable, 5)))
    if len(iterable) > 5:
        joined += ", ..."
    return joined


__join_dict_item = lambda item: ": ".join(map(repr, item))



from a_sync.a_sync.modifiers.manager cimport ModifierManager

cdef class _ModifiedMixin:
    cdef readonly ModifierManager modifiers
    cdef public object __wrapped__
    cdef object _asyncify(self, object func)

    cdef str __default
    cdef object __await
    cdef str get_default(self)
    cdef object get_await(self)

cdef void _validate_wrapped_fn(object fn)


from logging import Logger
from typing import AsyncIterator, Awaitable, Callable, Literal, NoReturn, TypeVar, overload

from typing_extensions import Concatenate, ParamSpec

from a_sync.a_sync.base import ASyncGenericBase
from a_sync.a_sync.method import ASyncBoundMethod
from a_sync.iter import ASyncGeneratorFunction


__P = ParamSpec("__P")
__T = TypeVar("__T")
__B = TypeVar("__B", bound=ASyncGenericBase)


_FIVE_MINUTES: Literal[300] = 300


@overload
def stuck_coro_debugger(
    fn: Callable[Concatenate[__B, __P], AsyncIterator[__T]],
    logger: Logger = logger,
    interval: int = _FIVE_MINUTES,
) -> ASyncGeneratorFunction[__P, __T]: ...


@overload
def stuck_coro_debugger(
    fn: Callable[Concatenate[__B, __P], Awaitable[__T]],
    logger: Logger = logger,
    interval: int = _FIVE_MINUTES,
) -> ASyncBoundMethod[__B, __P, __T]: ...


@overload
def stuck_coro_debugger(
    fn: Callable[Concatenate[__B, __P], __T], logger: Logger = logger, interval: int = _FIVE_MINUTES
) -> ASyncBoundMethod[__B, __P, __T]: ...


@overload
def stuck_coro_debugger(
    fn: Callable[__P, AsyncIterator[__T]], logger: Logger = logger, interval: int = _FIVE_MINUTES
) -> Callable[__P, AsyncIterator[__T]]: ...


@overload
def stuck_coro_debugger(
    fn: Callable[__P, Awaitable[__T]], logger: Logger = logger, interval: int = _FIVE_MINUTES
) -> Callable[__P, Awaitable[__T]]: ...


def stuck_coro_debugger(fn, logger=logger, interval=_FIVE_MINUTES): ...


async def _stuck_debug_task(
    logger: Logger, interval: int, fn: Callable[__P, __T], *args: __P.args, **kwargs: __P.kwargs
) -> NoReturn: ...


__all__ = ["stuck_coro_debugger"]
